//      
//  CUSTCAT.H
//
//  CComModule-derived class which implements registration of
//  custom categories through a Custom-Categories map.
//      
//  Usage:
//  1. Use any of the ATL wizards to generate the skeleton for
//     your COM application.
//  2. Make the following changes to stdafx.h:
//      - right under atlbase.h is included, add an include
//        statement to include custcat.h
//      - change the type of _Module from CComModule to
//        CCustCatComModule
//
//  3. Make the following changes to your project's main module
//     (the one where _Module is declared):
//      - change the type of _Module from CComModule to
//        CCustCatComModule
//      - write your custom category map. The custom category map
//        has the following syntax:
//
//      BEGIN_CUSTOMCATEGORY_MAP
//          CUSTOMCATEGORY( catid, description )
//          CUSTOMCATEGORY( another catid, another description )
//          etc etc
//      END_CUSTOMCATEGORY_MAP
//      
//  Written by Antonello Salvatucci
//      

#include <atlbase.h>

// Custom Category Map Definitions
typedef struct __MYATL_CUSTOMCATEGORY_MAP_ENTRY {
    int             itype;
    const CATID     *pcatid;
    LCID            lcid;
    OLECHAR         szdesc[128];  // same size as description field
                                  // of CATEGORYINFO structure
} MYATL_CUSTOMCATEGORY_MAP_ENTRY;

#define MYATL_CUSTOMCATEGORY_MAP_ENTRY_END              0
#define MYATL_CUSTOMCATEGORY_MAP_ENTRY_CUSTOM           1

const MYATL_CUSTOMCATEGORY_MAP_ENTRY *MyAtl_GetCustomCategoryMap();

#define BEGIN_CUSTOMCATEGORY_MAP        \
    const MYATL_CUSTOMCATEGORY_MAP_ENTRY \
            *MyAtl_GetCustomCategoryMap()  { \
    static const MYATL_CUSTOMCATEGORY_MAP_ENTRY pMap[] = {

#define CUSTOMCATEGORY( catid, description ) \
    { MYATL_CUSTOMCATEGORY_MAP_ENTRY_CUSTOM, &catid, 0, \
    L##description },

#define CUSTOMCATEGORY_LOCALE( catid, description, locale ) \
    { MYATL_CUSTOMCATEGORY_MAP_ENTRY_CUSTOM, &catid, locale, \
    L##description },

#define END_CUSTOMCATEGORY_MAP  \
    { MYATL_CUSTOMCATEGORY_MAP_ENTRY_END, NULL, 0, NULL } };   \
    return pMap; }

// Custom Category ComModule class
class CCustCatComModule : public CComModule
{
public:
    inline HRESULT RegisterServer(BOOL bRegTypeLib = FALSE, \
                                  const CLSID* pCLSID = NULL);
};


// -------------------------------------------------------------
// Helper functions
//      

//      
// Scans all the entries in the given custom category map and
// registers the category.
//      
inline HRESULT  MyAtlRegisterCustomCategoriesHelper( 
                const MYATL_CUSTOMCATEGORY_MAP_ENTRY* pcustcatmap,
                BOOL bRegister)
{
ICatRegister *                           pCatRegister;
HRESULT                                  hResult;
const MYATL_CUSTOMCATEGORY_MAP_ENTRY    *pEntry;
CATEGORYINFO                             catinfo;

    ATLASSERT(pcustcatmap);
    if( pcustcatmap == NULL )
        return( S_OK );

    // Get a ptr to ICatRegister
    hResult = CoCreateInstance(CLSID_StdComponentCategoriesMgr,
                                        NULL, CLSCTX_INPROC_SERVER,
                                        IID_ICatRegister,
                                        (void**)&pCatRegister);
    if( FAILED( hResult ) )
        return( hResult );

    // Scan category map and register all implemented categories
    hResult = S_OK;
    pEntry  = pcustcatmap;
    while( pEntry->itype != MYATL_CUSTOMCATEGORY_MAP_ENTRY_END ) {
        if( bRegister ) {
            if( pEntry->itype == \
                        MYATL_CUSTOMCATEGORY_MAP_ENTRY_CUSTOM ) {
                catinfo.catid   = *pEntry->pcatid;
                catinfo.lcid    = pEntry->lcid;
                memcpy(catinfo.szDescription, pEntry->szdesc, \
                        sizeof(catinfo.szDescription));

                hResult = pCatRegister->RegisterCategories( 1, \
                                                      &catinfo );
                if( FAILED( hResult ) )
                    break;
            }
        }
        pEntry++;
    }
    pCatRegister->Release();
    return( hResult );
}

// -------------------------------------------------------------
// CCustCatComModule Implementation
HRESULT CCustCatComModule::RegisterServer(BOOL bRegTypeLib, \
                                        const CLSID* pCLSID )
{
const MYATL_CUSTOMCATEGORY_MAP_ENTRY   *pEntry;
HRESULT                                 hRes = S_OK;

    // Register the custom categories
    pEntry  = ::MyAtl_GetCustomCategoryMap();
    hRes    = MyAtlRegisterCustomCategoriesHelper(pEntry, TRUE);

    //  Call default register server 
    if (SUCCEEDED(hRes))
        hRes = CComModule::RegisterServer(bRegTypeLib, pCLSID);

    return hRes;
}

// End of File
