#include <windows.h>
#include <windowsx.h>
#include <assert.h>
#include "enum.h"

class MyStringProvider : public IEnumString {
private:
    ULONG _cRef;
    ULONG _iCurr;
    ULONG _cNumStrings;
    LPCOLESTR *_pwszStrings;

public:
    MyStringProvider( LPCOLESTR *pwszStrings, ULONG cNumStrings );

    // IUnknown methods:
    STDMETHODIMP           QueryInterface ( REFIID, LPVOID * );
    STDMETHODIMP_( ULONG ) AddRef();
    STDMETHODIMP_( ULONG ) Release();

    // IEnumString methods:
    STDMETHODIMP Next( 
        ULONG celt, LPOLESTR *rgelt, ULONG *pceltFetched );
    STDMETHODIMP Skip( ULONG celt );
    STDMETHODIMP Reset( void );
    STDMETHODIMP Clone( IEnumString **ppenum );
};

MyStringProvider::MyStringProvider( 
    LPCOLESTR *pwszStrings, ULONG cNumStrings ) 
    : _pwszStrings( pwszStrings )
    , _cNumStrings( cNumStrings ), _cRef( 1 ), _iCurr( 0 )
{
}

STDMETHODIMP MyStringProvider::QueryInterface( 
    REFIID refIID, LPVOID *pInterface ) 
{
    if ( IsEqualGUID( refIID, IID_IUnknown ) ||
         IsEqualGUID( refIID, IID_IEnumString ) ) 
    {
        *pInterface = this;
        AddRef();
        return S_OK;
    }

    // Unsupported interface requested:
    *pInterface = 0;
    return E_NOINTERFACE;
}

STDMETHODIMP_( ULONG ) MyStringProvider::AddRef() {
    return ++_cRef;
}

STDMETHODIMP_( ULONG ) MyStringProvider::Release() {
    assert( 0 < _cRef );
    if ( 0 < --_cRef ) {
        return _cRef;
    }
    delete this; // Don't touch *any* members after this!
    return 0;
}

LPOLESTR my_strdup( LPCOLESTR pwsz ) {
    const UINT cch = wcslen( pwsz ) + 1;
    const UINT cb  = cch * sizeof( OLECHAR );
    LPOLESTR pwszRet = (LPOLESTR) CoTaskMemAlloc( cb );
    wcscpy( pwszRet, pwsz );
    assert( 0 == pwszRet[ cch - 1 ] );
    return pwszRet;
}

STDMETHODIMP MyStringProvider::Next( 
    ULONG celt, LPOLESTR *rgelt, ULONG *pceltFetched ) 
{
    assert( !IsBadWritePtr( rgelt, celt * sizeof *rgelt ) );
    assert( 0 <= _iCurr && _iCurr <= _cNumStrings );

    const ULONG nMaxElems = _cNumStrings - _iCurr;
    const ULONG ulFetched = __min( celt, nMaxElems );
    for ( ULONG i = 0; i < ulFetched; ++i ) {
        rgelt[ i ] = my_strdup( _pwszStrings[ _iCurr + i ] );
    }
    _iCurr += ulFetched;
    if ( 0 != *pceltFetched ) {
        *pceltFetched = ulFetched;
    }
    return ulFetched == celt ? S_OK : S_FALSE;
}

STDMETHODIMP MyStringProvider::Skip( ULONG celt ) {
    assert( 0 <= _iCurr );
    assert( _iCurr <= _cNumStrings );
    _iCurr += celt;
    if ( _cNumStrings < _iCurr ) {
        _iCurr = _cNumStrings;
        return S_FALSE;
    }
    return S_OK;
}

STDMETHODIMP MyStringProvider::Reset( void ) {
    _iCurr = 0;
    return S_OK;
}

STDMETHODIMP MyStringProvider::Clone( IEnumString ** ) {
    return E_NOTIMPL;
}

extern "C" IEnumString *
createStringProvider( LPCOLESTR *pwszStrings, ULONG cNumStrings ) {
    return new MyStringProvider( pwszStrings, cNumStrings );
}
